"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.guardRetType = void 0;
const config_js_1 = require("../../config.js");
const type_guard_js_1 = require("../../type-guard.js");
/**
 * Verify the TypeScript ret type is matching the NativeType from `RetType`
 */
function guardRetType(target, propertyKey, nativeType, pointerTypeList) {
    const designRetType = Reflect.getMetadata('design:returntype', target, propertyKey);
    config_js_1.log.verbose('Sidecar', 'guardRetType(%s.%s) designType/nativeType/pointerTypeList: %s/%s/[%s]', target.constructor.name, propertyKey, designRetType?.name ?? 'void', nativeType, pointerTypeList.join(','));
    try {
        (0, type_guard_js_1.guardNativeType)(nativeType)(designRetType);
        if (nativeType === 'pointer') {
            (0, type_guard_js_1.guardPointerType)(pointerTypeList)(designRetType);
        }
    }
    catch (e) {
        config_js_1.log.error('Sidecar', 'guardRetType() %s', e && e.message);
        throw new Error([
            `The ${target.constructor.name}.${String(propertyKey)}()`,
            `decorated by "@RetType(${nativeType}, ...)"`,
            `does match the design return type "${designRetType?.name ?? 'void'}"`,
        ].join('\n'));
    }
}
exports.guardRetType = guardRetType;
//# sourceMappingURL=guard-ret-type.js.map